@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * $Id: TestSuite.java,v 1.16 2000/08/15 21:37:55 metlov Exp $
+ * $Id: TestSuite.java,v 1.20 2000/08/30 17:14:32 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -30,20 +30,22 @@ public class TestSuite {
   public static void main(String[] args) {
 
     // Set UP Library
-    Class[] staticLib=new Class[3];
+    Class[] staticLib=new Class[2];
+    Class[] dotAllowedOn=new Class[1];
     try {
       staticLib[0]=Class.forName("java.lang.Math");
       VariableProvider tvp=new VariableProvider();
       staticLib[1]=tvp.getClass();
-      staticLib[2]=Class.forName("gnu.jel.StringLib");
+      dotAllowedOn[0]=Class.forName("java.lang.String");
+//      staticLib[2]=Class.forName("gnu.jel.StringLib");
     } catch(ClassNotFoundException e) {
       System.out.println(e);
       System.exit(1);
     };
-    Library lib=new Library(staticLib,null);
+    Library lib=new Library(staticLib,null,dotAllowedOn);
     try {
       lib.markStateDependent("random",null);
-    } catch (NoSuchMethodException e) {
+    } catch (CompilationException e) {
       // Can't be also
     };
 
@@ -228,8 +230,8 @@ public class TestSuite {
                    "(1==NaNf)||!(1!=NaNf)",
                    Boolean.FALSE,null,null,lib,o);
     
-    testExpression("indexOf(\"aaaca\",'c')+1==4 && "+
-                   "indexOf(1>2?\"cc\":\"aaaca\",'c')+1==4",
+    testExpression("\"aaaca\".indexOf('c')+1==4 && "+
+                   "(1>2?\"cc\":\"aaaca\").indexOf('c')+1==4",
                    Boolean.TRUE,null,null,lib,o);
 
     testExpression("round((float)4)",
@@ -246,11 +248,13 @@ public class TestSuite {
 		   
     testExpression("~0",new Integer(-1),null,null,lib,o);
 
-    testExpression("substring(\"abbb\"+\"ccc\"+'d'+(1>2?\"e\":\"d\"),1)",
+    testExpression("(\"abbb\"+\"ccc\"+'d'+(1>2?\"e\":\"d\")).substring(1)",
                    "bbbcccdd",null,null,lib,o);
 
-    testExpression("equals(substring(\"abbb\",1),\"bbb\")||false",
+    testExpression("\"abbb\".substring(1).equals(\"bbb\")||false",
                    Boolean.TRUE,null,null,lib,o);
+    //    testExpression("equals(substring(\"abbb\",1),\"bbb\")||false",
+    //    Boolean.TRUE,null,null,lib,o);
 
     o.println(); 
     o.println("----- TESTING VIRTUAL METHODS (x=5.0D) ------");
@@ -323,7 +327,21 @@ public class TestSuite {
 
     testExpression("arr[byteObj]", new Double(6.0), null, rtp, lib,o);
     testExpression("byteObj+1.0", new Double(3.0), null, rtp, lib,o);
-    
+
+    o.println(); o.println("----- DYNAMIC VARIABLES ------");
+    lib=new Library(staticLib,dynamicLib,dotLib,vp);
+    vp.addProperty("p1","p1value");
+    vp.addProperty("p1.s1","p1s1value");
+    vp.addProperty("p1.s2","p1s2value");
+    vp.addProperty("p1.d1",VariableProvider.makeJELDoubleObject(1.0));
+    vp.addProperty("p1.s2.ss1","p1s2ss1value");
+    testExpression("p1", "p1value", null, rtp, lib,o);
+    testExpression("p1.s1", "p1s1value", null, rtp, lib,o);
+    testExpression("p1.s2", "p1s2value", null, rtp, lib,o);
+    testExpression("p1.s2.ss1","p1s2ss1value", null, rtp, lib,o);
+    testExpression("p1.s2.length()", new Integer(9), null, rtp, lib,o);
+    testExpression("p1+(p1.d1+p1.s2.length()+1)", "p1value11.0", null, rtp, lib,o);
+
     o.println(); o.println("^^^^^^^^^^^^ SCORE success/failure = "+
                            successes+"/"+failures+
                            " ^^^^^^^^^^^^" );
