@@ -1,5 +1,5 @@
 /*-*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * $Id: VariableProvider.java,v 1.4 1999/12/19 17:43:31 metlov Exp $
+ * $Id: VariableProvider.java,v 1.6 2000/08/30 17:14:33 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -14,11 +14,37 @@
 
 
 
-public class VariableProvider {
+public class VariableProvider implements gnu.jel.DVResolver {
   public double xvar;
   
   public double x() {return xvar;};
 
+  // --- Dynamic variables testing section
+
+  private java.util.Hashtable properties=new java.util.Hashtable();
+
+  public gnu.jel.reflect.Double getDoubleProperty(String name) {
+    return (gnu.jel.reflect.Double) properties.get(name);
+  };
+
+  public String getStringProperty(String name) {
+    return (String) properties.get(name);
+  };
+
+  protected void addProperty(String name, Object value) {
+    properties.put(name,value);
+  };
+
+  public String getTypeName(String name) {
+    Object val=properties.get(name);
+    if (val==null) return null;
+    if (val instanceof String) return "String";
+    if (val instanceof gnu.jel.reflect.Double) return "Double";
+    return null;
+  };
+  
+  // --- END OF (Dynamic variables testing section)
+
   public double[] arr;
   
   public gnu.jel.reflect.Integer intObj;
@@ -57,8 +83,24 @@ public class VariableProvider {
     return new Double(d);
   };
 
+  public static gnu.jel.reflect.Double makeJELDoubleObject(double d) {
+    return new DoubleObject(d);
+  };
 };
 
+class DoubleObject implements gnu.jel.reflect.Double {
+  double val;
+  
+  public DoubleObject(double val) {
+	this.val=val;
+  };
+  
+  public double getValue() {
+	return val;
+  };;    
+};
+
+
 class IntegerObject implements gnu.jel.reflect.Integer {
   int val;
   
