@@ -1,5 +1,5 @@
 /*-*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * $Id: Library.java,v 1.5 2000/08/15 21:37:55 metlov Exp $
+ * $Id: Library.java,v 1.8 2000/08/30 17:14:32 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -59,6 +59,9 @@ public class Library {
   private Hashtable dotClasses; // Hashtable(classes, Hashtable(names, Hashtable(signatures, members)))
   private boolean noDotSecurity=false;
   
+  private DVResolver resolver;
+  private Class provider=null;
+
   /**
    * Creates a library for JEL.
    * <P> See the three argument constructor for more info.
@@ -71,6 +74,10 @@ public class Library {
     this(staticLib,dynamicLib,null);
   };
 
+  public Library(Class[] staticLib,Class[] dynamicLib,Class[] dotClasses) {
+    this(staticLib,dynamicLib,dotClasses,null);
+  };
+
   /**
    * Creates a library for JEL.
    * <P> The following should be kept in mind when constructing a library:
@@ -116,8 +123,13 @@ public class Library {
    *  methods are exported.
    * @param dotClasses is the array of classes on which the dot ('.')
    * operation is allowed.
+   * @param resolver is the object used to resolve the names.
    */
-  public Library(Class[] staticLib,Class[] dynamicLib,Class[] dotClasses) {
+  public Library(Class[] staticLib,Class[] dynamicLib,Class[] dotClasses,
+                 DVResolver resolver) {
+    this.resolver=resolver;
+    if (resolver!=null)
+      this.provider=dynamicLib[dynamicLib.length-1];
     if (dotClasses==null) {
       this.dotClasses=null;
     } else {
@@ -137,7 +149,6 @@ public class Library {
       rehash(staticLib,names,null,stateless);
     if (dynamicLib!=null)
       rehash(dynamicLib,names,dynIDs,null);
-
   };
 
   private void rehashDotClass(Class cls) {
@@ -196,11 +207,12 @@ public class Library {
       signatures.put(signature,m);
       return true;
     };
-    if (Debug.enabled) {
-      Debug.println("Conflict was detected during the library initialization."+
-                    " Conflicting "+"\""+name+signature+
-                    "\", conflicting :"+ conflicting_method+" and "+m+" .");
-    };
+//      if (Debug.enabled) {
+//        Debug.println("Conflict was detected during the library "+
+//                      "initialization."+
+//                      " Conflicting "+"\""+name+signature+
+//                      "\", conflicting :"+ conflicting_method+" and "+m+" .");
+//      };
     // If no debug then the method is ignored.
     return false;
   };
@@ -214,10 +226,10 @@ public class Library {
    * accidentally mark another function.
    * @param name is the function name.
    * @param params are the possible invocation parameters of the function.
-   * @exception NoSuchMethodException if the method can't be resolved   
+   * @exception CompilationException if the method can't be resolved   
    */
   public void markStateDependent(String name, Class[] params) 
-       throws NoSuchMethodException {
+       throws CompilationException {
     Object m=getMember(name,params);
     Object removed=stateless.remove(m);
     if (Debug.enabled)
@@ -234,6 +246,11 @@ public class Library {
   public boolean isStateless(Member o) {
     return stateless.containsKey(o);
   };
+
+  public String getDVTypeName(String name) {
+    if (resolver==null) return null;
+    return resolver.getTypeName(name);
+  };
   
   /**
    * Used to search library for method or field.
@@ -255,10 +272,10 @@ public class Library {
    * @param name is the name of the method to find.
    * @param params are the types of formal parameters in the method invocation.
    * @return the method/field object of the resolved method/field.
-   * @exception NoSuchMethodException if the method can't be resolved
+   * @exception CompilationException if the method can't be resolved
    */
   public Member getMember(String name,Class[] params) 
-    throws NoSuchMethodException {
+    throws CompilationException {
     return getMember(names,null,name,params);
   };
 
@@ -271,18 +288,18 @@ public class Library {
    * @param name is the name of the method to find.
    * @param params are the types of formal parameters in the method invocation.
    * @return the method/field object of the resolved method/field.
-   * @exception NoSuchMethodException if the method can't be resolved
+   * @exception CompilationException if the method can't be resolved
    */
   public Member getMember(Class container,String name,Class[] params) 
-    throws NoSuchMethodException {
+    throws CompilationException {
     // security
-    if (dotClasses==null)
-      throw new NoSuchMethodException("The access to class members is "+
-                                      "not allowed.");
-    else if (! (noDotSecurity || dotClasses.containsKey(container)))
-      throw new NoSuchMethodException("The access to members of "+
-                                      container.toString()+
-                                      " is not allowed.");
+    if (dotClasses==null) // dot operator is prohibited
+      throw new CompilationException(-1,11,null);
+    else if (! (noDotSecurity || dotClasses.containsKey(container))) {
+      // dot is not allowed in this particular class
+      Object[] paramsExc={container};
+      throw new CompilationException(-1,12,paramsExc);
+    };
     Hashtable hashedMembers=(Hashtable)dotClasses.get(container);
     if (hashedMembers==null) {
       rehashDotClass(container);
@@ -297,14 +314,13 @@ public class Library {
   // unlike Class.getMethod(...) tries various widening conversions.
   private static Member getMember(Hashtable hashedMembers, Class cls,
                                   String name,Class[] params) 
-    throws NoSuchMethodException {
+    throws CompilationException {
     Hashtable signatures=(Hashtable) hashedMembers.get(name);
 
-    if (signatures==null)
-      throw new NoSuchMethodException("The name \""+name+
-                                      "\" is not defined "+
-                                      (cls==null?"":"in the "+cls)+
-                                      " .");
+    if (signatures==null) { // name is not found
+      Object[] paramsExc={name,cls};
+      throw new CompilationException(-1,cls==null?5:6,paramsExc);
+    };
     
     // Choose applicable methods
     Vector applicable_methods=new Vector();
@@ -328,11 +344,8 @@ public class Library {
     };
     
     if (applicable_methods.size()==0) {
-      throw new NoSuchMethodException("Function \""+name+"\" exists"+
-                                      (cls==null?"":" in the "+cls)+
-                                      ", but parameters "+
-                                      describe(name,params)+
-                                      " can not be accepted by it.");
+      Object[] paramsExc={name,describe(name,params),cls};
+      throw new CompilationException(-1,cls==null?7:8,paramsExc);
     };
     
     if (applicable_methods.size()==1) 
@@ -360,13 +373,13 @@ public class Library {
         most_specific_params=cp;
       };
       
-      if (! (moreSpecific ^ lessSpecific))
-        throw new NoSuchMethodException("Ambiguity detected between \""+
-                                        describe(name,most_specific_params) +
-                                        "\" and \""+ describe(name,cp) +
-                                        "\" on invocation \""+
-                                        describe(name,params)+"\""+
-                                        (cls==null?"":" on the "+cls)+" .");
+      if (! (moreSpecific ^ lessSpecific)) {
+        Object[] paramsExc={describe(name,most_specific_params),
+                           describe(name,cp),
+                           describe(name,params),
+                           cls};
+        throw new CompilationException(-1,cls==null?9:10,paramsExc);        
+      };
     };
     return most_specific;
   };
@@ -628,7 +641,7 @@ public class Library {
       try {
         Member mf=ldot.getMember("size",params);
         t.testFail();
-      } catch (NoSuchMethodException exc) {
+      } catch (CompilationException exc) {
         t.testOK();
       };
     } catch (Throwable e) {
