@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- 
- * $Id: OPbinary.java,v 1.1 2000/02/11 22:08:18 metlov Exp $
+ * $Id: OPbinary.java,v 1.3 2000/08/28 20:05:53 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -24,36 +24,41 @@ public class OPbinary extends OPfunction {
   private int opsIDX; // index into ops array to get code for this op.
   private boolean strcat=false;
 
-  private static Method[] sb_append;
-  private static Method sb_append_string;
-  private static Class sb_class;
-  private static Class string_class;
+  private final static Method[] sb_append;
+  private final static Method sb_append_string;
+  private final static Class sb_class;
+  private final static Class string_class;
 
   private int paramcnt=0;
   
   static {
+    Class sb_class_1=null,string_class_1=null;
+    Method sb_append_string_1=null;
+    sb_append=new Method[9]; // up to references
     try {
-      sb_class=Class.forName("java.lang.StringBuffer");
-      string_class=Class.forName("java.lang.String");
+      sb_class_1=Class.forName("java.lang.StringBuffer");
+      string_class_1=Class.forName("java.lang.String");
 
       Class[] param=new Class[1];
-      param[0]=string_class;
-      sb_append_string=new Method(sb_class.getMethod("append",param));
+      param[0]=string_class_1;
+      sb_append_string_1=new Method(sb_class_1.getMethod("append",param));
       
-      sb_append=new Method[9]; // up to L
       param[0]=Class.forName("java.lang.Object");
-      sb_append[8]=new Method(sb_class.getMethod("append",param));
+      sb_append[8]=new Method(sb_class_1.getMethod("append",param));
  
       // -2 below excludes refs and voids
       for(int i=0;i<8;i++) { // Z B C S I J F D
         // byte is int, short is int
         param[0]=TypesStack.primitiveTypes[i==1?4:i==3?4:i];
-        sb_append[i]=new Method(sb_class.getMethod("append",param));
+        sb_append[i]=new Method(sb_class_1.getMethod("append",param));
       };
     } catch (Exception exc) {
       if (Debug.enabled)
         Debug.reportThrowable(exc);
     };
+    sb_class=sb_class_1;
+    string_class=string_class_1;
+    sb_append_string=sb_append_string_1;   
   };
   
   /**
@@ -171,7 +176,7 @@ public class OPbinary extends OPfunction {
    * @param code is the operation code
    */
   public OPbinary(TypesStack typesStk, Stack paramOPs, int code,OPlist list) 
-    throws IllegalStateException {
+    throws CompilationException {
     if (Debug.enabled)
       Debug.assert((code>=0) && (code<opNames.length) && 
                    (opNames.length==opSymbols.length));
@@ -203,10 +208,10 @@ public class OPbinary extends OPfunction {
       if ((code<14) || (code>16)) {
         // operation with binary promotion
         opsIDX=promotions[op1ID][op2ID];
-        if (opsIDX==255)
-          throw new IllegalStateException("Types "+op1Type+" and "+op2Type+
-                                          " are incompatible for "+
-                                          opNames[code]+" operation.");
+        if (opsIDX==255) { // types are incompatible (can't promote)
+          Object[] paramsExc={op1Type,op2Type,opNames[code]};
+          throw new CompilationException(-1,15,paramsExc);
+        };
         // insert conversion code
         if ((op1ID!=opsIDX) || (unwrap1OP)) {
           typesStk.pushID(op1ID);
@@ -221,11 +226,11 @@ public class OPbinary extends OPfunction {
           typesStk.pop();
         };
         
-        if (ops[code][opsIDX]==255)
-          throw new IllegalStateException("The "+opNames[code]+" operation"+
-                                          " is not defined for "+
-                                          TypesStack.primitiveTypeNames[resID]+
-                                          "s.");
+        if (ops[code][opsIDX]==255) { // operation is not defined on type
+          Object[] paramsExc={opNames[code],
+                              TypesStack.primitiveTypeNames[opsIDX]};
+          throw new CompilationException(-1,16,paramsExc);
+        };
         // determine the result type
         resID=opsIDX;
         // exception for comparizons where result is always boolean
@@ -249,21 +254,16 @@ public class OPbinary extends OPfunction {
                         new OPunary(typesStk,8,TypesStack.tsb_class,false));
           typesStk.pop();
         } else if (!TypesStack.isTSB(op1Type))
-          throw new IllegalStateException("First operand for string "+
-                                          "concatenation must be string.");
+          throw new CompilationException(-1,17,null);
         opsIDX=8;
         resID=8;
         resType=TypesStack.tsb_class;
         strcat=true;
       } else if (code==19) {  // array element access
         if (!op1Type.isArray())
-          throw new IllegalStateException("The first operand of the array "+
-                                          "element access operation must "+
-                                          "be an array.");
+          throw new CompilationException(-1,18,null);
         if (!TypesStack.isWidening(op2ID,4))
-          throw new IllegalStateException("Can't automatically convert "+
-                                          "array index of type "+op2Type+
-                                          " to int.");
+          throw new CompilationException(-1,19,op2Type);
         // convert the index to int if needed
         if ((op2ID!=4) || (unwrap2OP)) {
           typesStk.pushID(op2ID);
@@ -273,9 +273,7 @@ public class OPbinary extends OPfunction {
         resType=op1Type.getComponentType();
         resID=TypesStack.primitiveID(resType);
         opsIDX=resID;
-      } else throw new IllegalStateException("Operation \""+opNames[code]+
-                                             "\" is defined on primitive "+
-                                             "types only.");
+      } else throw new CompilationException(-1,20,opNames[code]);
     };
     typesStk.pushID(resID,resType);
   };
