@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- 
- * $Id: OPcall.java,v 1.2 2000/08/15 21:37:55 metlov Exp $
+ * $Id: OPcall.java,v 1.3 2000/08/28 20:05:53 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -38,7 +38,7 @@ public class OPcall extends OPfunction {
    */
   public OPcall(TypesStack typesStk, Stack paramOPs, Member m,
                 OPlist list, boolean attemptEval) 
-    throws IllegalStateException {
+    throws CompilationException {
     this.m=m;
     this.attemptEval=attemptEval;
     reqParamTypes=m.getParameterTypes();
