@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- 
- * $Id: OPunary.java,v 1.1 2000/02/11 22:08:18 metlov Exp $
+ * $Id: OPunary.java,v 1.3 2000/08/28 20:05:53 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -37,25 +37,27 @@ public class OPunary extends OPfunction {
   // 13 -- convert string to temporary string buffer (TSB)
   // 14 -- convert temporary string buffer (TSB) to string
 
-  private static Method sb_constructor;
-  private static Method sb_toString;
-  protected static Class sb_class;
-  protected static Class string_class;
+  private final static Method sb_constructor;
+  private final static Method sb_toString;
+  protected final static Class sb_class;
+  protected final static Class string_class;
   // methods to unwrap classes implementing gnu.jel.reflect.* interfaces
   private static Method[] unwrap; 
   
   static {
+    Method sb_constructor_1=null, sb_toString_1=null;
+    Class sb_class_1=null, string_class_1=null;
+    unwrap=new Method[8];    
     try {
-      sb_class=Class.forName("java.lang.StringBuffer");
-      string_class=Class.forName("java.lang.String");
+      sb_class_1=Class.forName("java.lang.StringBuffer");
+      string_class_1=Class.forName("java.lang.String");
       Class[] params=new Class[1];
-      params[0]=string_class;
-      sb_constructor=new Method(sb_class.getConstructor(params));
+      params[0]=string_class_1;
+      sb_constructor_1=new Method(sb_class_1.getConstructor(params));
       params=new Class[0];
-      sb_toString=new Method(sb_class.getMethod("toString",params));
+      sb_toString_1=new Method(sb_class_1.getMethod("toString",params));
       
       // prepare unwrapping methods
-      unwrap=new Method[8];
       for(int i=0;i<8;i++) 
         unwrap[i]=
           new Method(TypesStack.unwrapClasses[i].getMethod("getValue",
@@ -64,6 +66,10 @@ public class OPunary extends OPfunction {
       if (Debug.enabled)
         Debug.reportThrowable(e);
     };
+    sb_class=sb_class_1;
+    string_class=string_class_1;
+    sb_constructor=sb_constructor_1;
+    sb_toString=sb_toString_1;
   };
 
   protected final static
@@ -72,10 +78,10 @@ public class OPunary extends OPfunction {
 
   private final
     static int[][] una ={
-      // Z      B      C      S      I        J      F      D   L   VOID  OP/TO
-      {0x74,  0x74,  0xFF,  0x74,  0x74,    0x75,  0x76,  0x77,0xFF,0xFF},// NE
-      {0xFF,0x8202,0x8202,0x8202,0x8202,0x838502,  0xFF,  0xFF,0xFF,0xFF},// NO
-      {   0,  0xFF,  0xFF,  0xFF,  0xFF,    0xFF,  0xFF,  0xFF,0xFF,0xFF},//LNO
+      // Z      B      C      S      I        J      F      D   L   VOID OP/TO
+      {0x74,  0x74,  0xFF,  0x74,  0x74,    0x75,  0x76,  0x77,0xFF,0xFF},//NE
+      {0xFF,0x8202,0x8202,0x8202,0x8202,0x838502,  0xFF,  0xFF,0xFF,0xFF},//NO
+      {   0,  0xFF,  0xFF,  0xFF,  0xFF,    0xFF,  0xFF, 0xFF,0xFF,0xFF},//LNO
       {0x00,  0xFF,  0xFF,  0xFF,  0xFF,    0xFF,  0xFF,  0xFF,0xFF,0xFF},// Z
       {0x91,  0x00,  0x91,  0x91,  0x91,  0x9188,0x918B,0x918E,0XFF,0xFF},// B
       {0x92,  0x92,  0x00,  0x92,  0x92,  0x9288,0x928B,0x928E,0xFF,0xFF},// C
@@ -123,7 +129,7 @@ public class OPunary extends OPfunction {
    * @param cls class to convert to (used only if code==11)
    */
   public OPunary(TypesStack typesStk, int code) 
-    throws IllegalStateException {
+    throws CompilationException {
     if (Debug.enabled)
       Debug.assert((code>=0) && (code<=3));
 
@@ -132,12 +138,11 @@ public class OPunary extends OPfunction {
 
     int opTypeID=typesStk.peekID();
     Class opType=typesStk.peek();
-    if (una[code][opTypeID]==0xFF)
-      throw new IllegalStateException("The '"+opNames[code]+"' operation "+
-                                      "is not supported on "+
-                                      TypesStack.primitiveTypeNames[opTypeID]+
-                                      "s");
-
+    if (una[code][opTypeID]==0xFF) { // operation is not defined on types
+      Object[] paramsExc={opNames[code],opType};
+      throw new CompilationException(-1,16,paramsExc);
+    };
+    
     typesStk.pop();    
     if (code!=12) {       // ordinary ops do unary promotion
       resID=unary_prmtns[opTypeID]; 
@@ -159,7 +164,7 @@ public class OPunary extends OPfunction {
    * @param allownarrowing if narrowing conversions are allowed.
    */
   public OPunary(TypesStack typesStk, int targetID, Class targetClass,
-                 boolean allownarrowing) {
+                 boolean allownarrowing) throws CompilationException {
     if (Debug.enabled)
       Debug.assert((targetID>=0) && (targetID<=8) && 
                    ((targetID!=8) || (targetClass!=null)));
@@ -187,15 +192,15 @@ public class OPunary extends OPfunction {
       resType=TypesStack.primitiveTypes[targetID];
     };
     
-    if (code<0)
-      throw new IllegalStateException("Can not convert "+typesStk.peek()+
-                                      " to "+resType+" .");
+    if (code<0) { // can't convert at all
+      Object[] paramsExc={currClass,resType};
+      throw new CompilationException(-1,21,paramsExc);
+    };
     
-    if (!allownarrowing && !TypesStack.isWidening(currClass,resType))
-      throw new IllegalStateException("You must specify narrowing "+
-                                      "conversion "+
-                                      "from "+typesStk.peek()+
-                                      " to "+resType+" explicitly.");
+    if (!allownarrowing && !TypesStack.isWidening(currClass,resType)) {
+      Object[] paramsExc={currClass,resType};
+      throw new CompilationException(-1,22,paramsExc);
+    };
     typesStk.pop();
     typesStk.pushID(resID,resType);
   };
@@ -207,7 +212,7 @@ public class OPunary extends OPfunction {
    * @param allownarrowing if narrowing conversions are allowed.
    */
   public OPunary(TypesStack typesStk, Class cls,
-                 boolean allownarrowing) {
+                 boolean allownarrowing) throws CompilationException {
     this(typesStk,TypesStack.primitiveID(cls),cls,allownarrowing);
   };
 
