@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- 
- * $Id: TypesStack.java,v 1.1 2000/02/11 22:08:18 metlov Exp $
+ * $Id: TypesStack.java,v 1.3 2000/08/28 22:00:00 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -364,6 +364,13 @@ public class TypesStack implements Cloneable {
     return c==tsb_class;
   };
 
+  /**
+   * If the given class c.isTSB returns java.lang.String otherwise c.
+   */
+  public static Class normalizeTSB(Class c) {
+    return (c==tsb_class?string_class:c);
+  };
+
   protected static Number widen(Object o, int clsID) {
     switch (clsID) {
     case 0: // Z
